package com.mdd.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.yulichang.query.MPJQueryWrapper;
import com.mdd.admin.service.IFinanceRechargerService;
import com.mdd.admin.validate.commons.PageValidate;
import com.mdd.admin.validate.finance.FinanceRechargeSearchValidate;
import com.mdd.admin.vo.finance.FinanceRechargeListVo;
import com.mdd.common.config.GlobalConfig;
import com.mdd.common.core.PageResult;
import com.mdd.common.entity.RechargeOrder;
import com.mdd.common.entity.RefundLog;
import com.mdd.common.entity.RefundRecord;
import com.mdd.common.entity.user.User;
import com.mdd.common.enums.LogMoneyEnum;
import com.mdd.common.enums.PaymentEnum;
import com.mdd.common.enums.RefundEnum;
import com.mdd.common.exception.OperateException;
import com.mdd.common.mapper.RechargeOrderMapper;
import com.mdd.common.mapper.RefundLogMapper;
import com.mdd.common.mapper.RefundRecordMapper;
import com.mdd.common.mapper.log.LogMoneyMapper;
import com.mdd.common.mapper.user.UserMapper;
import com.mdd.common.plugin.wechat.WxPayDriver;
import com.mdd.common.plugin.wechat.request.RefundRequestV3;
import com.mdd.common.util.AmountUtil;
import com.mdd.common.util.StringUtils;
import com.mdd.common.util.TimeUtils;
import com.mdd.common.util.UrlUtils;
import java.util.Iterator;
import javax.annotation.Resource;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.util.Assert;

@Service
public class FinanceRechargerServiceImpl implements IFinanceRechargerService {
   @Resource
   RechargeOrderMapper rechargeOrderMapper;
   @Resource
   UserMapper userMapper;
   @Resource
   LogMoneyMapper logMoneyMapper;
   @Resource
   RefundRecordMapper refundRecordMapper;
   @Resource
   RefundLogMapper refundLogMapper;
   @Resource
   DataSourceTransactionManager transactionManager;
   @Resource
   TransactionDefinition transactionDefinition;

   public PageResult<FinanceRechargeListVo> list(PageValidate pageValidate, FinanceRechargeSearchValidate searchValidate) {
      Integer pageNo = pageValidate.getPageNo();
      Integer pageSize = pageValidate.getPageSize();
      MPJQueryWrapper<RechargeOrder> mpjQueryWrapper = new MPJQueryWrapper();
      ((MPJQueryWrapper)mpjQueryWrapper.selectAll(RechargeOrder.class).select(new String[]{"U.id as user_id,U.sn as user_sn,U.nickname,U.avatar"}).leftJoin("?_user U ON U.id=t.user_id".replace("?_", GlobalConfig.tablePrefix))).orderByDesc("id");
      this.rechargeOrderMapper.setSearch(mpjQueryWrapper, searchValidate, new String[]{"like:sn@t.order_sn:str", "=:payWay@t.pay_way:int", "=:payStatus@t.pay_status:int", "datetime:startTime-endTime@create_time:long"});
      if (StringUtils.isNotEmpty(searchValidate.getKeyword())) {
         String keyword = searchValidate.getKeyword();
         mpjQueryWrapper.nested((wq) -> {
            MPJQueryWrapper var10000 = (MPJQueryWrapper)((MPJQueryWrapper)((MPJQueryWrapper)((MPJQueryWrapper)((MPJQueryWrapper)wq.like("U.nickname", keyword)).or()).like("U.sn", keyword)).or()).like("U.mobile", keyword);
         });
      }

      IPage<FinanceRechargeListVo> iPage = this.rechargeOrderMapper.selectJoinPage(new Page((long)pageNo, (long)pageSize), FinanceRechargeListVo.class, mpjQueryWrapper);
      Iterator var7 = iPage.getRecords().iterator();

      while(var7.hasNext()) {
         FinanceRechargeListVo vo = (FinanceRechargeListVo)var7.next();
         vo.setCreateTime(TimeUtils.timestampToDate(vo.getCreateTime()));
         vo.setPayTime(TimeUtils.timestampToDate(vo.getPayTime()));
         vo.setAvatar(UrlUtils.toAbsoluteUrl(vo.getAvatar()));
         vo.setPayWay(PaymentEnum.getPayWayMsg(Integer.parseInt(vo.getPayWay())));
         vo.setIsRefund(0);
         if (vo.getPayStatus().equals(1)) {
            RefundRecord refundRecord = (RefundRecord)this.refundRecordMapper.selectOne((Wrapper)((QueryWrapper)((QueryWrapper)(new QueryWrapper()).eq("order_type", "recharge")).eq("order_id", vo.getId())).last("limit 1"));
            if (StringUtils.isNotNull(refundRecord)) {
               vo.setIsRefund(1);
               vo.setRefundStatusMsg(RefundEnum.getRefundStatusMsg(refundRecord.getRefundStatus()));
            }
         }
      }

      return PageResult.iPageHandle(iPage);
   }

   public void refund(Integer orderId, Integer adminId) {
      RechargeOrder rechargeOrder = (RechargeOrder)this.rechargeOrderMapper.selectById(orderId);
      Assert.notNull(rechargeOrder, "充值订单不存在!");
      if (!rechargeOrder.getPayStatus().equals(PaymentEnum.OK_PAID.getCode())) {
         throw new OperateException("当前订单不可退款!");
      } else if (rechargeOrder.getRefundStatus().equals(1)) {
         throw new OperateException("订单已发起退款,退款失败请到退款记录重新退款!");
      } else {
         User user = (User)this.userMapper.selectById(rechargeOrder.getUserId());
         if (user.getMoney().compareTo(rechargeOrder.getOrderAmount()) < 0) {
            throw new OperateException("退款失败:用户余额已不足退款金额!");
         } else {
            TransactionStatus transactionStatus = this.transactionManager.getTransaction(this.transactionDefinition);
            RefundRecord refundRecord = null;
            RefundLog log = null;

            try {
               rechargeOrder.setRefundStatus(1);
               this.rechargeOrderMapper.updateById(rechargeOrder);
               user.setMoney(user.getMoney().subtract(rechargeOrder.getOrderAmount()));
               this.userMapper.updateById(user);
               this.logMoneyMapper.dec(user.getId(), LogMoneyEnum.UM_DEC_RECHARGE.getCode(), rechargeOrder.getOrderAmount(), rechargeOrder.getId(), rechargeOrder.getOrderSn(), "充值订单退款", (String)null);
               String refundSn = this.refundRecordMapper.randMakeOrderSn("sn");
               refundRecord = new RefundRecord();
               refundRecord.setSn(refundSn);
               refundRecord.setUserId(rechargeOrder.getUserId());
               refundRecord.setOrderId(rechargeOrder.getId());
               refundRecord.setOrderSn(rechargeOrder.getOrderSn());
               refundRecord.setOrderType(RefundEnum.getOrderType(RefundEnum.ORDER_TYPE_RECHARGE.getCode()));
               refundRecord.setOrderAmount(rechargeOrder.getOrderAmount());
               refundRecord.setRefundAmount(rechargeOrder.getOrderAmount());
               refundRecord.setRefundType(RefundEnum.TYPE_ADMIN.getCode());
               refundRecord.setTransactionId(refundRecord.getTransactionId());
               refundRecord.setRefundWay(rechargeOrder.getPayWay());
               this.refundRecordMapper.insert(refundRecord);
               log = new RefundLog();
               log.setSn(this.refundLogMapper.randMakeOrderSn("sn"));
               log.setRecordId(refundRecord.getId());
               log.setUserId(rechargeOrder.getUserId());
               log.setHandleId(adminId);
               log.setOrderAmount(rechargeOrder.getOrderAmount());
               log.setRefundAmount(refundRecord.getRefundAmount());
               log.setRefundStatus(RefundEnum.REFUND_ING.getCode());
               this.refundLogMapper.insert(log);
               RefundRequestV3 requestV3 = new RefundRequestV3();
               requestV3.setTransactionId(rechargeOrder.getTransactionId());
               requestV3.setOutTradeNo(rechargeOrder.getOrderSn());
               requestV3.setOutRefundNo(refundSn);
               requestV3.setTotalAmount(AmountUtil.yuan2Fen(rechargeOrder.getOrderAmount().toString()));
               requestV3.setRefundAmount(AmountUtil.yuan2Fen(rechargeOrder.getOrderAmount().toString()));
               WxPayDriver.refund(requestV3);
               refundRecord.setRefundStatus(RefundEnum.REFUND_SUCCESS.getCode());
               this.refundRecordMapper.updateById(refundRecord);
               log.setRefundStatus(RefundEnum.REFUND_SUCCESS.getCode());
               this.refundLogMapper.updateById(log);
               this.transactionManager.commit(transactionStatus);
            } catch (Exception var10) {
               Exception e = var10;
               this.transactionManager.rollback(transactionStatus);
               if (StringUtils.isNotNull(refundRecord)) {
                  refundRecord.setRefundStatus(RefundEnum.REFUND_ERROR.getCode());
                  this.refundRecordMapper.updateById(refundRecord);
               }

               if (StringUtils.isNotNull(log)) {
                  log.setRefundStatus(RefundEnum.REFUND_ERROR.getCode());
                  this.refundLogMapper.updateById(log);
               }

               throw new OperateException(e.getMessage());
            }
         }
      }
   }

   public void refundAgain(Integer recordId, Integer adminId) {
      TransactionStatus transactionStatus = this.transactionManager.getTransaction(this.transactionDefinition);
      RefundLog log = null;

      try {
         RefundRecord refundRecord = (RefundRecord)this.refundRecordMapper.selectById(recordId);
         RechargeOrder rechargeOrder = (RechargeOrder)this.rechargeOrderMapper.selectById(refundRecord.getOrderId());
         Assert.notNull(rechargeOrder, "充值订单丢失!");
         log = (RefundLog)this.refundLogMapper.selectOne((Wrapper)((QueryWrapper)(new QueryWrapper()).eq("record_id", recordId)).last("limit 1"));
         log.setRefundStatus(RefundEnum.REFUND_ING.getCode());
         this.refundLogMapper.updateById(log);
         RefundRequestV3 requestV3 = new RefundRequestV3();
         requestV3.setTransactionId(refundRecord.getTransactionId());
         requestV3.setOutTradeNo(refundRecord.getOrderSn());
         requestV3.setOutRefundNo(refundRecord.getSn());
         requestV3.setTotalAmount(AmountUtil.yuan2Fen(rechargeOrder.getOrderAmount().toString()));
         requestV3.setRefundAmount(AmountUtil.yuan2Fen(refundRecord.getOrderAmount().toString()));
         WxPayDriver.refund(requestV3);
         log.setRefundStatus(RefundEnum.REFUND_SUCCESS.getCode());
         this.refundLogMapper.updateById(log);
         this.transactionManager.commit(transactionStatus);
      } catch (Exception var8) {
         Exception e = var8;
         this.transactionManager.rollback(transactionStatus);
         if (StringUtils.isNotNull(log)) {
            log.setRefundStatus(RefundEnum.REFUND_ERROR.getCode());
            this.refundLogMapper.updateById(log);
         }

         throw new OperateException(e.getMessage());
      }
   }
}
